@@ -4,47 +4,46 @@ var google, django, gettext;
 
 (function () {
     var jQuery = jQuery || $ || django.jQuery;
+    /* Add a new selector to jQuery that excludes parent items which match a
+       given selector */
+    jQuery.expr[':'].parents = function(a, i, m) {
+        return jQuery(a).parents(m[3]).length < 1;
+    };
+
     jQuery(function ($) {
         function getGroupedTranslationFields() {
             /** Returns a grouped set of all text based model translation fields.
              * The returned datastructure will look something like this:
              * {
-             *   'title': {
-             *     'en': HTMLInputElement,
-             *     'de': HTMLInputElement,
-             *     'fr': HTMLInputElement
-             *   },
-             *   'body': {
-             *     'en': HTMLTextAreaElement,
-             *     'de': HTMLTextAreaElement,
-             *     'fr': HTMLTextAreaElement
-             *   }
+             *     'title': {
+             *         'en': HTMLInputElement,
+             *         'de': HTMLInputElement,
+             *         'zh_tw': HTMLInputElement
+             *     },
+             *     'body': {
+             *         'en': HTMLTextAreaElement,
+             *         'de': HTMLTextAreaElement,
+             *         'zh_tw': HTMLTextAreaElement
+             *     }
              * }
              */
-            var translation_fields = $('.modeltranslation').filter('input[type=text]:visible, textarea:visible'),
+            var translation_fields = $('.mt').filter(
+                'input[type=text]:visible, textarea:visible').filter(
+                ':parents(.tabular)'), // exclude tabular inlines
               grouped_translations = {};
 
             translation_fields.each(function (i, el) {
-                /*
-                // FIXME: Fails if there's an inline which has the same field name as
-                //        the edited object.
-
-                // Extract fieldname and original language code from class attribute
-                var css_lang_suffix = 'modeltranslation-field-';
-                var name = '';
-                var lang = '';
+                var field_prefix = 'mt-field-',
+                    name = '',
+                    lang = '';
                 $.each($(el).attr('class').split(' '), function(j, cls) {
-                    if (cls.substring(0, css_lang_suffix.length) === css_lang_suffix) {
-                        var v = cls.substring(css_lang_suffix.length,
-                                              cls.length).split('__');
+                    if (cls.substring(0, field_prefix.length) === field_prefix) {
+                        var v = cls.substring(field_prefix.length,
+                                              cls.length).split('-');
                         name = v[0];
                         lang = v[1];
                     }
                 });
-                */
-                var name = $(el).attr('name').split('_'),
-                  lang = name.pop();
-                name = name.join('_');
                 if (!grouped_translations[name]) {
                     grouped_translations[name] = {};
                 }
@@ -80,7 +79,7 @@ var google, django, gettext;
                     }
                     container.find('script').remove();
                     panel = $('<div id="' + id + '"></div>').append(container);
-                    tab = $('<li' + (label.hasClass('required') ? ' class="required"' : '') + '><a href="#' + id + '">' + lang + '</a></li>');
+                    tab = $('<li' + (label.hasClass('required') ? ' class="required"' : '') + '><a href="#' + id + '">' + lang.replace('_', '-') + '</a></li>');
                     tabs_list.append(tab);
                     tabs_container.append(panel);
                 });
@@ -103,7 +102,7 @@ var google, django, gettext;
                 });
             });
             $.each(unique_languages, function (i, language) {
-                select.append($('<option value="' + i + '">' + language + '</option>'));
+                select.append($('<option value="' + i + '">' + language.replace('_', '-') + '</option>'));
             });
             select.change(function (e) {
                 $.each(tabs, function (i, tab) {
